/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 **/

#include <string>

#include "storage/StorageBlockInfo.hpp"
#include "utility/StringUtil.hpp"

using std::string;

namespace quickstep {

string BlockIdUtil::ToString(const block_id block) {
  string block_str("(");
  block_str.append(ToZeroPaddedString(Domain(block), kBlockIdDomainLengthInDigits));
  block_str.append(", ");
  block_str.append(ToZeroPaddedString(Counter(block), kBlockIdCounterLengthInDigits));
  block_str.append(")");

  return block_str;
}

const char *kTupleStorageSubBlockTypeNames[] = {
  "BasicColumnStore",
  "CompressedPackedRowStore",
  "CompressedColumnStore",
  "SplitRowStore"
};

const char *kIndexSubBlockTypeNames[] = {
  "CSBTree",
  "SMA",
};

}  // namespace quickstep
